@@ -26,12 +26,12 @@
 
 #include <klocalizedstring.h>
 
-#include <QSpinBox>
-#include <QPainter>
-#include <QLabel>
-#include <QGridLayout>
 #include <QCheckBox>
-
+#include <QHBoxLayout>
+#include <QLabel>
+#include <QPainter>
+#include <QSpinBox>
+#include <QVBoxLayout>
 
 KoCreatePathTool::KoCreatePathTool(KoCanvasBase *canvas)
     : KoToolBase(*(new KoCreatePathToolPrivate(this, canvas)))
@@ -201,6 +201,8 @@ void KoCreatePathTool::mousePressEvent(KoPointerEvent *event)
             }
         }
     } else {
+        beginShape();
+
         KoPathShape *pathShape = new KoPathShape();
         d->shape = pathShape;
         pathShape->setShapeId(KoPathShapeId);
@@ -386,6 +388,7 @@ void KoCreatePathTool::endPath()
 
     d->addPathShape();
     repaintDecorations();
+    endShape();
 }
 
 void KoCreatePathTool::endPathWithoutLastPoint()
@@ -398,6 +401,7 @@ void KoCreatePathTool::endPathWithoutLastPoint()
 
         repaintDecorations();
     }
+    endShape();
 }
 
 void KoCreatePathTool::cancelPath()
@@ -410,6 +414,7 @@ void KoCreatePathTool::cancelPath()
     }
     d->cleanUp();
     repaintDecorations();
+    endShape();
 }
 
 void KoCreatePathTool::removeLastPoint()
@@ -534,37 +539,58 @@ QList<QPointer<QWidget> > KoCreatePathTool::createOptionWidgets()
 
     QList<QPointer<QWidget> > list;
 
-    QCheckBox *smoothCurves = new QCheckBox(i18n("Autosmooth curve"));
+    QWidget *widget = new QWidget();
+    widget->setObjectName("bezier-curve-tool-widget");
+    widget->setWindowTitle(i18n("Path options"));
+
+    QCheckBox *smoothCurves = new QCheckBox(i18n("Autosmooth curve"), widget);
     smoothCurves->setObjectName("smooth-curves-widget");
     smoothCurves->setChecked(d->autoSmoothCurves);
-    connect(smoothCurves, SIGNAL(toggled(bool)), this, SLOT(autoSmoothCurvesChanged(bool)));
-    connect(this, SIGNAL(sigUpdateAutoSmoothCurvesGUI(bool)), smoothCurves, SLOT(setChecked(bool)));
 
-    list.append(smoothCurves);
+    QCheckBox *angleSnap = new QCheckBox(i18n("Activate angle snap"), widget);
+    angleSnap->setObjectName("angle-snap-widget");
+    angleSnap->setChecked(false);
+    angleSnap->setCheckable(true);
 
-    QWidget *angleWidget = new QWidget();
-    angleWidget->setObjectName("Angle Constraints");
-    QGridLayout *layout = new QGridLayout(angleWidget);
-    layout->addWidget(new QLabel(i18n("Angle snapping delta:"), angleWidget), 0, 0);
-    KisAngleSelector *angleEdit = new KisAngleSelector(angleWidget);
+    KisAngleSelector *angleEdit = new KisAngleSelector(widget);
+    angleEdit->setObjectName("angle-edit-widget");
     angleEdit->setAngle(d->angleSnappingDelta);
     angleEdit->setRange(1, 360);
     angleEdit->setDecimals(0);
     angleEdit->setFlipOptionsMode(KisAngleSelector::FlipOptionsMode_MenuButton);
-    layout->addWidget(angleEdit, 0, 1);
-    layout->addWidget(new QLabel(i18n("Activate angle snap:"), angleWidget), 1, 0);
-    QCheckBox *angleSnap = new QCheckBox(angleWidget);
-    angleSnap->setChecked(false);
-    angleSnap->setCheckable(true);
-    layout->addWidget(angleSnap, 1, 1);
-    QWidget *specialSpacer = new QWidget();
-    specialSpacer->setObjectName("SpecialSpacer");
-    layout->addWidget(specialSpacer, 2, 1);
-    angleWidget->setWindowTitle(i18n("Angle Constraints"));
-    list.append(angleWidget);
+    angleEdit->setEnabled(angleSnap->isChecked());
 
+    QHBoxLayout *angleEditLayout = new QHBoxLayout;
+    angleEditLayout->setContentsMargins(10, 0, 0, 0);
+    angleEditLayout->setSpacing(0);
+    angleEditLayout->addWidget(angleEdit);
+
+    QVBoxLayout *mainLayout = new QVBoxLayout;
+    mainLayout->setContentsMargins(0, 0, 0, 0);
+    mainLayout->setSpacing(5);
+
+    mainLayout->addWidget(smoothCurves);
+    mainLayout->addWidget(angleSnap);
+    mainLayout->addLayout(angleEditLayout);
+
+    widget->setLayout(mainLayout);
+
+    list.append(widget);
+
+    connect(smoothCurves,
+            SIGNAL(toggled(bool)),
+            this,
+            SLOT(autoSmoothCurvesChanged(bool)));
+    connect(this,
+            SIGNAL(sigUpdateAutoSmoothCurvesGUI(bool)),
+            smoothCurves,
+            SLOT(setChecked(bool)));
     connect(angleEdit, SIGNAL(angleChanged(qreal)), this, SLOT(angleDeltaChanged(qreal)));
     connect(angleSnap, SIGNAL(stateChanged(int)), this, SLOT(angleSnapChanged(int)));
+    connect(angleSnap,
+            SIGNAL(toggled(bool)),
+            angleEdit,
+            SLOT(setEnabled(bool)));
 
     return list;
 }
