@@ -19,7 +19,7 @@ class KoShapeControllerBase;
 class KoToolFactoryBase;
 class KoCanvasBase;
 class KoToolBase;
-class KActionCollection;
+class KisKActionCollection;
 class KoShape;
 class KoShapeLayer;
 class QKeySequence;
@@ -140,7 +140,7 @@ public:
      * @param ac the actionCollection that will be the parent of the actions.
      * @param controller tools registered with this controller will have all their actions added as well.
      */
-    void registerToolActions(KActionCollection *ac, KoCanvasController *controller);
+    void registerToolActions(KisKActionCollection *ac, KoCanvasController *controller);
 
     /**
      * Register a new canvas controller
