@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2015 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2015 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_idle_watcher.h"
@@ -26,7 +14,6 @@
 
 struct KisIdleWatcher::Private
 {
-    static const int IDLE_CHECK_PERIOD = 200; /* ms */
     static const int IDLE_CHECK_COUNT = 4; /* ticks */
 
     Private(int delay, KisIdleWatcher *q)
@@ -35,7 +22,7 @@ struct KisIdleWatcher::Private
           idleCheckCounter(0)
     {
         idleCheckTimer.setSingleShot(true);
-        idleCheckTimer.setInterval(IDLE_CHECK_PERIOD);
+        idleCheckTimer.setInterval(delay);
     }
 
     KisSignalAutoConnectionsStore connectionsStore;
