@@ -143,6 +143,12 @@ QString KisFolderStorage::resourceMd5(const QString &url)
     return result;
 }
 
+QString KisFolderStorage::resourceFilePath(const QString &url)
+{
+    QFileInfo file(location() + "/" + url);
+    return file.exists() ? file.absoluteFilePath() : QString();
+}
+
 QSharedPointer<KisResourceStorage::ResourceIterator> KisFolderStorage::resources(const QString &resourceType)
 {
     QVector<VersionedResourceEntry> entries;
@@ -193,9 +199,9 @@ bool KisFolderStorage::importResource(const QString &url, QIODevice *device)
     if (f.exists()) return result;
 
     if (f.open(QFile::WriteOnly)) {
-        quint64 writtenBytes = f.write(device->readAll());
+        qint64 writtenBytes = f.write(device->readAll());
         f.close();
-        result = writtenBytes == device->size();
+        result = (writtenBytes == device->size());
     } else {
         qWarning() << "Cannot open" << resourcesLocation << "for writing";
     }
@@ -243,18 +249,18 @@ bool KisFolderStorage::addResource(const QString &resourceType, KoResourceSP res
         return false;
     }
 
-    QFile f(fi.absoluteFilePath());
-    if (!f.open(QFile::WriteOnly)) {
+    QFile resourceFile(fi.absoluteFilePath());
+    if (!resourceFile.open(QFile::WriteOnly)) {
         qWarning() << "Could not open" << fi.absoluteFilePath() << "for writing.";
         return false;
     }
 
-    if (!resource->saveToDevice(&f)) {
+    if (!resource->saveToDevice(&resourceFile)) {
         qWarning() << "Could not save resource to" << fi.absoluteFilePath();
-        f.close();
+        resourceFile.close();
         return false;
     }
-    f.close();
+    resourceFile.close();
 
 
 
