@@ -29,11 +29,12 @@
 #include "ResourceDebug.h"
 #include <kis_assert.h>
 
+#include <KisCppQuirks.h>
+
 const QString dbDriver = "QSQLITE";
 
-const QString KisResourceCacheDb::dbLocationKey { "ResourceCacheDbDirectory" };
 const QString KisResourceCacheDb::resourceCacheDbFilename { "resourcecache.sqlite" };
-const QString KisResourceCacheDb::databaseVersion { "0.0.16" };
+const QString KisResourceCacheDb::databaseVersion { "0.0.17" };
 QStringList KisResourceCacheDb::storageTypes { QStringList() };
 QStringList KisResourceCacheDb::disabledBundles { QStringList() << "Krita_3_Default_Resources.bundle" };
 
@@ -174,12 +175,15 @@ QSqlError createDatabase(const QString &location)
                 schemaIsOutDated = true;
                 KBackup::numberedBackupFile(location + "/" + KisResourceCacheDb::resourceCacheDbFilename);
 
-                if (newSchemaVersionNumber == QVersionNumber::fromString("0.0.16")
+                if (newSchemaVersionNumber == QVersionNumber::fromString("0.0.17")
                         && QVersionNumber::compare(oldSchemaVersionNumber, QVersionNumber::fromString("0.0.14")) > 0
-                        && QVersionNumber::compare(oldSchemaVersionNumber, QVersionNumber::fromString("0.0.16")) < 0) {
+                        && QVersionNumber::compare(oldSchemaVersionNumber, QVersionNumber::fromString("0.0.17")) < 0) {
                     bool from14to15 = oldSchemaVersionNumber == QVersionNumber::fromString("0.0.14");
                     bool from15to16 = oldSchemaVersionNumber == QVersionNumber::fromString("0.0.14")
                             || oldSchemaVersionNumber == QVersionNumber::fromString("0.0.15");
+                    bool from16to17 = oldSchemaVersionNumber == QVersionNumber::fromString("0.0.14")
+                            || oldSchemaVersionNumber == QVersionNumber::fromString("0.0.15")
+                            || oldSchemaVersionNumber == QVersionNumber::fromString("0.0.16");
 
                     bool success = true;
                     if (from14to15) {
@@ -217,6 +221,23 @@ QSqlError createDatabase(const QString &location)
                         }
                     }
 
+                    if (from16to17) {
+                        qWarning() << "Going to update resource signature index";
+
+                        QFile f(":/create_index_resources_signature.sql");
+                        if (f.open(QFile::ReadOnly)) {
+                            QSqlQuery q;
+                            if (!q.exec(f.readAll())) {
+                                qWarning() << "Could not create index for resources signature" << q.lastError();
+                                return db.lastError();
+                            }
+                            infoResources << "Created resources signature index";
+                        }
+                        else {
+                            return QSqlError("Error executing SQL", QString("Could not find SQL file for resources signature index"), QSqlError::StatementError);
+                        }
+                    }
+
                     if (success) {
                         if (!updateSchemaVersion()) {
                             return QSqlError("Error executing SQL", QString("Could not update schema version."), QSqlError::StatementError);
@@ -270,7 +291,13 @@ QSqlError createDatabase(const QString &location)
     }
 
     // Create indexes
-    QStringList indexes = QStringList() << "storages" << "versioned_resources" << "tags" << "resources" << "tag_translations" << "resource_tags";
+    QStringList indexes;
+
+    // these indexes came in version 0.0.16
+    indexes << "storages" << "versioned_resources" << "tags" << "resources" << "tag_translations" << "resource_tags";
+
+    // this indexes came in version 0.0.17
+    indexes << "resources_signature";
 
     Q_FOREACH(const QString &index, indexes) {
         QFile f(":/create_index_" + index + ".sql");
@@ -1609,6 +1636,7 @@ struct ResourceVersion : public boost::less_than_comparable<ResourceVersion>
 
 };
 
+MAYBE_UNUSED
 QDebug operator<<(QDebug dbg, const ResourceVersion &ver)
 {
     dbg.nospace() << "ResourceVersion("
@@ -1791,7 +1819,7 @@ bool KisResourceCacheDb::synchronizeStorage(KisResourceStorageSP storage)
             KoResourceSP res = storage->resource(itA->url);
 
             if (!res) {
-                KisUsageLogger::log("Could not load resource" + itA->url);
+                KisUsageLogger::log("Could not load resource " + itA->url);
                 ++itA;
                 continue;
             }
@@ -1799,21 +1827,21 @@ bool KisResourceCacheDb::synchronizeStorage(KisResourceStorageSP storage)
             res->setVersion(itA->version);
             res->setMD5Sum(storage->resourceMd5(itA->url));
             if (!res->valid()) {
-                KisUsageLogger::log("Could not retrieve md5 for resource" + itA->url);
+                KisUsageLogger::log("Could not retrieve md5 for resource " + itA->url);
                 ++itA;
                 continue;
             }
 
             const bool retval = addResource(storage, itA->timestamp, res, resourceType);
             if (!retval) {
-                KisUsageLogger::log("Could not add resource" + itA->url);
+                KisUsageLogger::log("Could not add resource " + itA->url);
                 ++itA;
                 continue;
             }
 
             const int resourceId = res->resourceId();
             KIS_SAFE_ASSERT_RECOVER(resourceId >= 0) {
-                KisUsageLogger::log("Could not get id for resource" + itA->url);
+                KisUsageLogger::log("Could not get id for resource " + itA->url);
                 ++itA;
                 continue;
             }
@@ -1829,7 +1857,7 @@ bool KisResourceCacheDb::synchronizeStorage(KisResourceStorageSP storage)
 
                 const bool retval = addResourceVersion(resourceId, it->timestamp, storage, res);
                 KIS_SAFE_ASSERT_RECOVER(retval) {
-                    KisUsageLogger::log("Could not add version for resource" + itA->url);
+                    KisUsageLogger::log("Could not add version for resource " + itA->url);
                     continue;
                 }
             }
