@@ -63,7 +63,7 @@
 #include <generator/kis_generator.h>
 #include <brushengine/kis_paintop_registry.h>
 #include <kis_meta_data_io_backend.h>
-#include "kisexiv2/kis_exiv2.h"
+#include <kis_meta_data_backend_registry.h>
 #include "KisApplicationArguments.h"
 #include <kis_debug.h>
 #include "kis_action_registry.h"
@@ -85,6 +85,7 @@
 #include <KisResourceLoader.h>
 #include <KisResourceLoaderRegistry.h>
 
+#include <KisBrushTypeMetaDataFixup.h>
 #include <kis_gbr_brush.h>
 #include <kis_png_brush.h>
 #include <kis_svg_brush.h>
@@ -206,7 +207,7 @@ KisApplication::KisApplication(const QString &key, int &argc, char **argv)
     setWindowIcon(KisIconUtils::loadIcon("krita-branding"));
 
     if (qgetenv("KRITA_NO_STYLE_OVERRIDE").isEmpty()) {
-        QStringList styles = QStringList() << "breeze" << "fusion";
+        QStringList styles = QStringList() << "macintosh" /*<< "breeze" */<< "fusion";
         if (!styles.contains(style()->objectName().toLower())) {
             Q_FOREACH (const QString & style, styles) {
                 if (!setStyle(style)) {
@@ -265,6 +266,7 @@ BOOL isWow64()
 
 void KisApplication::initializeGlobals(const KisApplicationArguments &args)
 {
+    Q_UNUSED(args)
     // There are no globals to initialize from the arguments now. There used
     // to be the `dpi` argument, but it doesn't do anything anymore.
 }
@@ -272,23 +274,23 @@ void KisApplication::initializeGlobals(const KisApplicationArguments &args)
 void KisApplication::addResourceTypes()
 {
     // All Krita's resource types
-    KoResourcePaths::addResourceType("markers", "data", "/styles/");
-    KoResourcePaths::addResourceType("kis_pics", "data", "/pics/");
-    KoResourcePaths::addResourceType("kis_images", "data", "/images/");
-    KoResourcePaths::addResourceType("metadata_schema", "data", "/metadata/schemas/");
-    KoResourcePaths::addResourceType("gmic_definitions", "data", "/gmic/");
-    KoResourcePaths::addResourceType("kis_shortcuts", "data", "/shortcuts/");
-    KoResourcePaths::addResourceType("kis_actions", "data", "/actions");
-    KoResourcePaths::addResourceType("kis_actions", "data", "/pykrita");
-    KoResourcePaths::addResourceType("icc_profiles", "data", "/color/icc");
-    KoResourcePaths::addResourceType("icc_profiles", "data", "/profiles/");
-    KoResourcePaths::addResourceType(ResourceType::FilterEffects, "data", "/effects/");
-    KoResourcePaths::addResourceType("tags", "data", "/tags/");
-    KoResourcePaths::addResourceType("templates", "data", "/templates");
-    KoResourcePaths::addResourceType("pythonscripts", "data", "/pykrita");
-    KoResourcePaths::addResourceType("preset_icons", "data", "/preset_icons");
+    KoResourcePaths::addAssetType("markers", "data", "/styles/");
+    KoResourcePaths::addAssetType("kis_pics", "data", "/pics/");
+    KoResourcePaths::addAssetType("kis_images", "data", "/images/");
+    KoResourcePaths::addAssetType("metadata_schema", "data", "/metadata/schemas/");
+    KoResourcePaths::addAssetType("gmic_definitions", "data", "/gmic/");
+    KoResourcePaths::addAssetType("kis_shortcuts", "data", "/shortcuts/");
+    KoResourcePaths::addAssetType("kis_actions", "data", "/actions");
+    KoResourcePaths::addAssetType("kis_actions", "data", "/pykrita");
+    KoResourcePaths::addAssetType("icc_profiles", "data", "/color/icc");
+    KoResourcePaths::addAssetType("icc_profiles", "data", "/profiles/");
+    KoResourcePaths::addAssetType(ResourceType::FilterEffects, "data", "/effects/");
+    KoResourcePaths::addAssetType("tags", "data", "/tags/");
+    KoResourcePaths::addAssetType("templates", "data", "/templates");
+    KoResourcePaths::addAssetType("pythonscripts", "data", "/pykrita");
+    KoResourcePaths::addAssetType("preset_icons", "data", "/preset_icons");
 #if defined HAVE_SEEXPR
-    KoResourcePaths::addResourceType(ResourceType::SeExprScripts, "data", "/seexpr_scripts/", true);
+    KoResourcePaths::addAssetType(ResourceType::SeExprScripts, "data", "/seexpr_scripts/", true);
 #endif
 
     // Make directories for all resources we can save, and tags
@@ -358,11 +360,9 @@ bool KisApplication::registerResources()
                                                      i18nc("Resource type name", "Layer styles"),
                                                      QStringList() << "application/x-photoshop-style"));
 
-    KConfigGroup cfg(KSharedConfig::openConfig(), "");
-    QString databaseLocation = cfg.readEntry(KisResourceCacheDb::dbLocationKey, "");
-    if (databaseLocation.isEmpty()) {
-        databaseLocation = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
-    }
+    reg->registerFixup(10, new KisBrushTypeMetaDataFixup());
+    QString databaseLocation = KoResourcePaths::getAppDataLocation();
+
 
     if (!KisResourceCacheDb::initialize(databaseLocation)) {
         QMessageBox::critical(qApp->activeWindow(), i18nc("@title:window", "Krita: Fatal error"), i18n("%1\n\nKrita will quit now.", KisResourceCacheDb::lastError()));
@@ -390,15 +390,7 @@ void KisApplication::loadPlugins()
     KisPaintOpRegistry::instance();
     KoToolRegistry::instance();
     KoDockRegistry::instance();
-}
-
-void KisApplication::loadGuiPlugins()
-{
-    // XXX_EXIV: make the exiv io backends real plugins
-    setSplashScreenLoadingText(i18n("Loading Plugins Exiv/IO..."));
-    processEvents();
-    //    qDebug() << "loading exiv2";
-    KisExiv2::initialize();
+    KisMetadataBackendRegistry::instance();
 }
 
 bool KisApplication::start(const KisApplicationArguments &args)
@@ -471,9 +463,6 @@ bool KisApplication::start(const KisApplicationArguments &args)
         return false;
     }
 
-    // Load the gui plugins
-    loadGuiPlugins();
-
     KisPart *kisPart = KisPart::instance();
     if (needsMainWindow) {
         // show a mainWindow asap, if we want that
@@ -971,9 +960,9 @@ bool KisApplication::createNewDocFromTemplate(const QString &fileName, KisMainWi
         QString desktopName(fileName);
         const QString templatesResourcePath =  QStringLiteral("templates/");
 
-        QStringList paths = KoResourcePaths::findAllResources("data", templatesResourcePath + "*/" + desktopName);
+        QStringList paths = KoResourcePaths::findAllAssets("data", templatesResourcePath + "*/" + desktopName);
         if (paths.isEmpty()) {
-            paths = KoResourcePaths::findAllResources("data", templatesResourcePath + desktopName);
+            paths = KoResourcePaths::findAllAssets("data", templatesResourcePath + desktopName);
         }
 
         if (paths.isEmpty()) {
