@@ -38,6 +38,7 @@
 #include <kis_paint_device.h>
 #include <KisPart.h>
 #include <KisKineticScroller.h>
+#include "KisMainWindow.h"
 
 #include <utils/KisUpdaterBase.h>
 
@@ -69,8 +70,8 @@
 #ifdef Q_OS_ANDROID
 #include <QtAndroid>
 
-QPushButton* KisWelcomePageWidget::donationLink;
-QLabel* KisWelcomePageWidget::donationBannerImage;
+QPushButton* KisWelcomePageWidget::donationLink {nullptr};
+QLabel* KisWelcomePageWidget::donationBannerImage {nullptr};
 #endif
 
 #ifdef Q_OS_WIN
@@ -191,17 +192,14 @@ KisWelcomePageWidget::KisWelcomePageWidget(QWidget *parent)
 
     verticalLayout->addWidget(donationBannerImage);
 
-    jboolean bannerPurchased = QAndroidJniObject::callStaticMethod<jboolean>("org/krita/android/DonationHelper", "isBadgePurchased", "()Z");
-    if (bannerPurchased) {
-        donationLink->hide();
-        donationBannerImage->show();
-        QAndroidJniObject::callStaticMethod<void>("org/krita/android/DonationHelper", "endConnection", "()V");
-    } else {
-        donationLink->show();
-        donationBannerImage->hide();
-    }
-#endif
+    donationLink->show();
+    donationBannerImage->hide();
 
+    // this will asynchronously lead to donationSuccessful (i.e if it *is* successful) which will hide the
+    // link and enable the donation banner.
+    QAndroidJniObject::callStaticMethod<void>("org/krita/android/DonationHelper", "checkBadgePurchased",
+                                              "()V");
+#endif
 
     // configure the News area
     KisConfig cfg(true);
@@ -288,10 +286,11 @@ void KisWelcomePageWidget::setMainWindow(KisMainWindow* mainWin)
         // allows RSS news items to apply analytics tracking.
         newsWidget->setAnalyticsTracking("?" + analyticsString);
 
-        KisRecentDocumentsModelWrapper *recentFilesModel = mainWin->recentFilesModel();
+        KisRecentDocumentsModelWrapper *recentFilesModel = KisRecentDocumentsModelWrapper::instance();
         connect(recentFilesModel, SIGNAL(sigModelIsUpToDate()),
                 this, SLOT(slotRecentFilesModelIsUpToDate()));
         recentDocumentsListView->setModel(&recentFilesModel->model());
+        slotRecentFilesModelIsUpToDate();
     }
 }
 
@@ -430,7 +429,7 @@ void KisWelcomePageWidget::dragEnterEvent(QDragEnterEvent *event)
 {
     showDropAreaIndicator(true);
     if (event->mimeData()->hasUrls() ||
-        event->mimeData()->hasFormat("application/x-krita-node") ||
+        event->mimeData()->hasFormat("application/x-krita-node-internal-pointer") ||
         event->mimeData()->hasFormat("application/x-qt-image")) {
         return event->accept();
     }
@@ -451,7 +450,7 @@ void KisWelcomePageWidget::dropEvent(QDropEvent *event)
                 }
             } else if (!url.isLocalFile()) {
                 QScopedPointer<QTemporaryFile> tmp(new QTemporaryFile());
-                tmp->setAutoRemove(true);
+                tmp->setFileName(url.fileName());
 
                 KisRemoteFileFetcher fetcher;
 
@@ -474,7 +473,7 @@ void KisWelcomePageWidget::dragMoveEvent(QDragMoveEvent *event)
     m_mainWindow->dragMoveEvent(event);
 
     if (event->mimeData()->hasUrls() ||
-        event->mimeData()->hasFormat("application/x-krita-node") ||
+        event->mimeData()->hasFormat("application/x-krita-node-internal-pointer") ||
         event->mimeData()->hasFormat("application/x-qt-image")) {
         return event->accept();
     }
@@ -661,7 +660,7 @@ void KisWelcomePageWidget::slotOpenFileClicked()
 
 void KisWelcomePageWidget::slotRecentFilesModelIsUpToDate()
 {
-    KisRecentDocumentsModelWrapper *recentFilesModel = m_mainWindow->recentFilesModel();
+    KisRecentDocumentsModelWrapper *recentFilesModel = KisRecentDocumentsModelWrapper::instance();
     const bool modelIsEmpty = recentFilesModel->model().rowCount() == 0;
 
     if (modelIsEmpty) {
