@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2006 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2006 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_node_shape.h"
@@ -141,12 +129,4 @@ void KisNodeShape::paint(QPainter &, KoShapePaintingContext &) const
 {
 }
 
-void KisNodeShape::saveOdf(KoShapeSavingContext &) const
-{
-}
-
-bool KisNodeShape::loadOdf(const KoXmlElement &, KoShapeLoadingContext &)
-{
-    return false;
-}
 
