@@ -1,20 +1,7 @@
 /*
- *  Copyright (c) 2011 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2011 Boudewijn Rempt <boud@valdyas.org>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "kis_mimedata.h"
@@ -31,6 +18,11 @@
 #include "kis_shape_controller.h"
 #include "KisPart.h"
 #include "kis_layer_utils.h"
+#include "kis_generator_registry.h"
+#include "KisGlobalResourcesInterface.h"
+#include "kis_filter_configuration.h"
+#include "kis_generator_layer.h"
+#include "kis_selection.h"
 #include "kis_node_insertion_adapter.h"
 #include "kis_dummies_facade_base.h"
 #include "kis_node_dummies_graph.h"
@@ -76,7 +68,10 @@ void KisMimeData::deepCopyNodes()
     {
         KisImageBarrierLockerWithFeedbackAllowNull locker(m_image);
         Q_FOREACH (KisNodeSP node, m_nodes) {
-            newNodes << safeCopyNode(node);
+            KisNodeSP newNode = safeCopyNode(node);
+            newNode->setImage(nullptr);
+
+            newNodes << newNode;
         }
     }
 
@@ -93,9 +88,7 @@ QStringList KisMimeData::formats () const
 {
     QStringList f = QMimeData::formats();
     if (m_nodes.size() > 0) {
-        f << "application/x-krita-node"
-          << "application/x-krita-node-url"
-          << "application/x-qt-image"
+        f << "application/x-qt-image"
           << "application/zip"
           << "application/x-krita-node-internal-pointer";
     }
@@ -104,10 +97,7 @@ QStringList KisMimeData::formats () const
 
 KisDocument *createDocument(QList<KisNodeSP> nodes, KisImageSP srcImage)
 {
-    qDebug() << nodes;
-    qDebug() << srcImage;
-
-    KisDocument *doc = KisPart::instance()->createDocument();
+    KisDocument *doc = KisPart::instance()->createTemporaryDocument();
     QRect rc;
     Q_FOREACH (KisNodeSP node, nodes) {
         if (node) {
@@ -115,6 +105,9 @@ KisDocument *createDocument(QList<KisNodeSP> nodes, KisImageSP srcImage)
         }
     }
 
+    QRect offset(0, 0, rc.width(), rc.height());
+    rc |= offset;
+
     if (rc.isEmpty() && srcImage) {
         rc = srcImage->bounds();
     }
@@ -122,10 +115,22 @@ KisDocument *createDocument(QList<KisNodeSP> nodes, KisImageSP srcImage)
     KisImageSP image = new KisImage(0, rc.width(), rc.height(), nodes.first()->colorSpace(), nodes.first()->name());
     image->setAllowMasksOnRootNode(true);
 
+    if (srcImage) {
+        image->setResolution(srcImage->xRes(), srcImage->yRes());
+    }
+
     {
         KisImageBarrierLockerWithFeedbackAllowNull locker(srcImage);
         Q_FOREACH (KisNodeSP node, nodes) {
-            image->addNode(safeCopyNode(node));
+            KisNodeSP clonedNode = safeCopyNode(node);
+            /// HACK ALERT: here we just initilize parent image link
+            ///             and skip initilizing shapeController!
+            ///             Ideally, we should call initializeExternalNode()
+            ///             instead.
+            image->addNode(clonedNode);
+            KIS_SAFE_ASSERT_RECOVER(clonedNode->image() == KisImageWSP(image)) {
+                clonedNode->setImage(image);
+            }
         }
     }
 
@@ -169,32 +174,12 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
                                                            KoColorConversionTransformation::internalRenderingIntent(),
                                                            KoColorConversionTransformation::internalConversionFlags());
     }
-    else if (mimetype == "application/x-krita-node" ||
-             mimetype == "application/zip") {
+    else if (mimetype == "application/zip") {
 
         QByteArray ba = serializeToByteArray(m_nodes, m_image);
         return ba;
 
     }
-    else if (mimetype == "application/x-krita-node-url") {
-
-        QByteArray ba = serializeToByteArray(m_nodes, m_image);
-
-        QString temporaryPath =
-                QDir::tempPath() + '/' +
-                QString("krita_tmp_dnd_layer_%1_%2.kra")
-                .arg(QApplication::applicationPid())
-                .arg(qrand());
-
-
-        QFile file(temporaryPath);
-        file.open(QFile::WriteOnly);
-        file.write(ba);
-        file.flush();
-        file.close();
-
-        return QUrl::fromLocalFile(temporaryPath).toEncoded();
-    }
     else if (mimetype == "application/x-krita-node-internal-pointer") {
 
         QDomDocument doc("krita_internal_node_pointer");
@@ -219,16 +204,24 @@ QVariant KisMimeData::retrieveData(const QString &mimetype, QVariant::Type prefe
 }
 
 void KisMimeData::initializeExternalNode(KisNodeSP *node,
-                                         KisImageWSP image,
+                                         KisImageSP srcImage,
+                                         KisImageSP dstImage,
                                          KisShapeController *shapeController)
 {
-    // adjust the link to a correct image object
-    (*node)->setImage(image);
-
     KisShapeLayer *shapeLayer = dynamic_cast<KisShapeLayer*>(node->data());
     if (shapeLayer) {
         // attach the layer to a new shape controller
         KisShapeLayer *shapeLayer2 = new KisShapeLayer(*shapeLayer, shapeController);
+
+        if (!qFuzzyCompare(dstImage->xRes(), srcImage->xRes()) ||
+            !qFuzzyCompare(dstImage->yRes(), srcImage->yRes())) {
+
+            const QTransform t = QTransform::fromScale(srcImage->xRes() / dstImage->xRes(),
+                                                       srcImage->yRes() / dstImage->yRes());
+
+            shapeLayer2->setTransformation(shapeLayer2->transformation() * t);
+        }
+
         *node = shapeLayer2;
     }
 }
@@ -288,7 +281,7 @@ QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
             if ((forceCopy || copyNode) && sourceImage == image) {
                 KisLayerUtils::addCopyOfNameTag(node);
             }
-            initializeExternalNode(&node, image, shapeController);
+            initializeExternalNode(&node, sourceImage, image, shapeController);
             clones << node;
         }
         nodes = clones;
@@ -297,55 +290,30 @@ QList<KisNodeSP> KisMimeData::tryLoadInternalNodes(const QMimeData *data,
     return nodes;
 }
 
-QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
-                                        const QRect &imageBounds,
-                                        const QPoint &preferredCenter,
-                                        bool forceRecenter,
-                                        KisImageWSP image,
-                                        KisShapeController *shapeController)
+QList<KisNodeSP> KisMimeData::loadNonNativeNodes(const QMimeData *data,
+                                                KisImageWSP image,
+                                                KisShapeController *shapeController)
 {
-    bool alwaysRecenter = false;
+    bool doRecenter = false;
     QList<KisNodeSP> nodes;
 
-    if (data->hasFormat("application/x-krita-node")) {
-        KisDocument *tempDoc = KisPart::instance()->createDocument();
-        QByteArray ba = data->data("application/x-krita-node");
-        QBuffer buf(&ba);
-        KisImportExportFilter *filter = tempDoc->importExportManager()->filterForMimeType(tempDoc->nativeFormatMimeType(), KisImportExportManager::Import);
-        filter->setBatchMode(true);
-        bool result = (filter->convert(tempDoc, &buf).isOk());
-
-        if (result) {
-            KisImageWSP tempImage = tempDoc->image();
-            Q_FOREACH (KisNodeSP node, tempImage->root()->childNodes(QStringList(), KoProperties())) {
-                tempImage->removeNode(node);
-                initializeExternalNode(&node, image, shapeController);
-                nodes << node;
-            }
+    if (nodes.isEmpty() && (data->hasFormat("application/x-color") || data->hasFormat("krita/x-colorsetentry"))) {
+        QColor color = data->hasColor() ? qvariant_cast<QColor>(data->colorData()) : QColor(255, 0, 255);
+        if (!data->hasColor() && data->hasFormat("krita/x-colorsetentry")) {
+            QByteArray byteData = data->data("krita/x-colorsetentry");
+            KisSwatch s = KisSwatch::fromByteArray(byteData);
+            color = s.color().toQColor();
         }
-        delete filter;
-        delete tempDoc;
-    }
+        KisGeneratorSP generator = KisGeneratorRegistry::instance()->value("color");
+        KisFilterConfigurationSP defaultConfig = generator->factoryConfiguration(KisGlobalResourcesInterface::instance());
+        defaultConfig->setProperty("color", color);
+        defaultConfig->createLocalResourcesSnapshot(KisGlobalResourcesInterface::instance());
 
-    if (nodes.isEmpty() && data->hasFormat("application/x-krita-node-url")) {
-        QByteArray ba = data->data("application/x-krita-node-url");
-        QUrl url = QUrl::fromEncoded(ba);
-        Q_ASSERT(url.isLocalFile());
-
-        KisDocument *tempDoc = KisPart::instance()->createDocument();
-        bool result = tempDoc->openUrl(url);
-
-        if (result) {
-            KisImageSP tempImage = tempDoc->image();
-            Q_FOREACH (KisNodeSP node, tempImage->root()->childNodes(QStringList(), KoProperties())) {
-                tempImage->removeNode(node);
-                initializeExternalNode(&node, image, shapeController);
-                nodes << node;
-            }
+        if (image) {
+            KisGeneratorLayerSP fillLayer = new KisGeneratorLayer(image, image->nextLayerName(i18n("Fill Layer")),
+                                                                  defaultConfig, image->globalSelection());
+            nodes << fillLayer;
         }
-
-        delete tempDoc;
-        QFile::remove(url.toLocalFile());
     }
 
     if (nodes.isEmpty() && data->hasImage()) {
@@ -358,17 +326,16 @@ QList<KisNodeSP> KisMimeData::loadNodes(const QMimeData *data,
             nodes << new KisPaintLayer(image.data(), image->nextLayerName(), OPACITY_OPAQUE_U8, device);
         }
 
-        alwaysRecenter = true;
+        doRecenter = true;
     }
 
-    if (!nodes.isEmpty()) {
-        Q_FOREACH (KisNodeSP node, nodes) {
-            QRect bounds = node->projection()->exactBounds();
-            if (alwaysRecenter || forceRecenter ||
-                    (!imageBounds.contains(bounds) &&
-                     !imageBounds.intersects(bounds))) {
+    if (!nodes.isEmpty() && doRecenter) {
+        const QRect imageBounds = image->bounds();
 
-                QPoint pt = preferredCenter - bounds.center();
+        Q_FOREACH (KisNodeSP node, nodes) {
+            const QRect layerBounds = node->projection()->exactBounds();
+            if (doRecenter) {
+                QPoint pt = imageBounds.center() - layerBounds.center();
                 node->setX(pt.x());
                 node->setY(pt.y());
             }
@@ -420,7 +387,8 @@ bool correctNewNodeLocation(KisNodeList nodes,
     KisNodeSP parentNode = parentDummy->node();
     bool result = true;
 
-    if(!nodeAllowsAsChild(parentDummy->node(), nodes)) {
+    if(!nodeAllowsAsChild(parentDummy->node(), nodes) ||
+            (parentDummy->node()->inherits("KisGroupLayer") && parentDummy->node()->collapsed())) {
         aboveThisDummy = parentDummy;
         parentDummy = parentDummy->parent();
 
@@ -444,11 +412,7 @@ KisNodeList KisMimeData::loadNodesFast(
                                           copyNode /* IN-OUT */);
 
     if (nodes.isEmpty()) {
-        QRect imageBounds = image->bounds();
-        nodes = KisMimeData::loadNodes(data,
-                                       imageBounds, imageBounds.center(),
-                                       false,
-                                       image, shapeController);
+        nodes = KisMimeData::loadNonNativeNodes(data, image, shapeController);
         /**
          * Don't try to move a node originating from another image,
          * just copy it.
@@ -459,16 +423,44 @@ KisNodeList KisMimeData::loadNodesFast(
     return nodes;
 }
 
+KisNodeList KisMimeData::loadNodesFastAndRecenter(const QPoint &preferredCenter, const QMimeData *data, KisImageSP image, KisShapeController *shapeController, bool &copyNode)
+{
+    KisNodeList nodes = loadNodesFast(data, image, shapeController, copyNode);
+
+    Q_FOREACH (KisNodeSP node, nodes) {
+        const QRect layerBounds = node->exactBounds();
+        const QPoint offset = preferredCenter - layerBounds.center();
+
+        node->setX(node->x() + offset.x());
+        node->setY(node->y() + offset.y());
+    }
+
+    return nodes;
+}
+
 bool KisMimeData::insertMimeLayers(const QMimeData *data,
                                    KisImageSP image,
                                    KisShapeController *shapeController,
                                    KisNodeDummy *parentDummy,
                                    KisNodeDummy *aboveThisDummy,
                                    bool copyNode,
-                                   KisNodeInsertionAdapter *nodeInsertionAdapter)
+                                   KisNodeInsertionAdapter *nodeInsertionAdapter,
+                                   bool changeOffset,
+                                   QPointF offset)
 {
     QList<KisNodeSP> nodes = loadNodesFast(data, image, shapeController, copyNode /* IN-OUT */);
 
+    if (changeOffset) {
+        Q_FOREACH (KisNodeSP node, nodes) {
+            KisLayerUtils::recursiveApplyNodes(node, [offset] (KisNodeSP node){
+                if (node->hasEditablePaintDevice()) {
+                    KisPaintDeviceSP dev = node->paintDevice();
+                    dev->moveTo(offset.x(), offset.y());
+                }
+            });
+        }
+    }
+
     if (nodes.isEmpty()) return false;
 
     bool result = true;
@@ -489,5 +481,20 @@ bool KisMimeData::insertMimeLayers(const QMimeData *data,
         nodeInsertionAdapter->moveNodes(nodes, parentDummy->node(), aboveThisNode);
     }
 
+    const bool hasDelayedNodes =
+        std::find_if(nodes.begin(), nodes.end(),
+                     [] (KisNodeSP node) {
+                         return bool(dynamic_cast<KisDelayedUpdateNodeInterface*>(node.data()));
+                     }) != nodes.end();
+
+    if (hasDelayedNodes) {
+        /**
+         * We have the node juggler running, so it will delay the update of the
+         * generator layers that might be included into the paste. To avoid
+         * that we should forcefully to make it stop
+         */
+        image->requestStrokeEnd();
+    }
+
     return result;
 }
