@@ -44,7 +44,7 @@ public:
         MASKING_BRUSH
     };
 
-    KisPaintOpOption(KisPaintOpOption::PaintopCategory category, bool checked);
+    KisPaintOpOption(QString label, KisPaintOpOption::PaintopCategory category, bool checked);
     ~KisPaintOpOption() override;
 
     KisPaintOpOption::PaintopCategory category() const;
@@ -56,6 +56,8 @@ public:
     void setLocked(bool value);
     bool isLocked() const;
 
+    QString label() const;
+
     /**
      * Reimplement this to use the image in the option widget
      */
@@ -112,8 +114,8 @@ Q_SIGNALS:
 
 protected:
 
-    bool m_checkable;
-    bool m_locked;
+    bool m_checkable {false};
+    bool m_locked {false};
 
 private:
 
