@@ -1,21 +1,9 @@
 /* This file is part of the Calligra project
- * Copyright (C) 2005 Thomas Zander <zander@kde.org>
- * Copyright (C) 2005 C. Boemann <cbo@boemann.dk>
- * Copyright (C) 2007 Boudewijn Rempt <boud@valdyas.org>
+ * SPDX-FileCopyrightText: 2005 Thomas Zander <zander@kde.org>
+ * SPDX-FileCopyrightText: 2005 C. Boemann <cbo@boemann.dk>
+ * SPDX-FileCopyrightText: 2007 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "widgets/kis_custom_image_widget.h"
@@ -26,7 +14,6 @@
 #include <QComboBox>
 #include <QRect>
 #include <QApplication>
-#include <QClipboard>
 #include <QDesktopWidget>
 #include <QDialogButtonBox>
 #include <QFile>
@@ -36,6 +23,7 @@
 #include <KoResourcePaths.h>
 
 #include <KFormat>
+#include <kstandardguiitem.h>
 
 #include <kis_debug.h>
 
@@ -58,11 +46,22 @@
 
 #include "kis_config.h"
 #include "KisPart.h"
-#include "kis_clipboard.h"
 #include "KisDocument.h"
 #include "widgets/kis_cmb_idlist.h"
 #include <KisSqueezedComboBox.h>
+#include "kis_signals_blocker.h"
 
+static QString pixelsInchStr()
+{
+    static QString str = i18n("Pixels/Inch");
+    return str;
+}
+
+static const QString pixelsCentimeterStr()
+{
+    static QString str = i18n("Pixels/Centimeter");
+    return str;
+}
 
 KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qint32 defHeight, double resolution, const QString& defColorModel, const QString& defColorDepth, const QString& defColorProfile, const QString& imageName)
     : WdgNewImage(parent)
@@ -86,15 +85,18 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     cmbHeightUnit->addItems(KoUnit::listOfUnitNameForUi(KoUnit::ListAll));
     cmbHeightUnit->setCurrentIndex(m_heightUnit.indexInListForUi(KoUnit::ListAll));
 
+    cmbResolutionUnit->addItem(pixelsInchStr());
+    cmbResolutionUnit->addItem(pixelsCentimeterStr());
+
     doubleResolution->setValue(72.0 * resolution);
-    doubleResolution->setDecimals(0);
+    doubleResolution->setDecimals(2);
 
     imageGroupSpacer->changeSize(0, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
     grpClipboard->hide();
 
     sliderOpacity->setRange(0, 100, 0);
     sliderOpacity->setValue(100);
-    sliderOpacity->setSuffix("%");
+    sliderOpacity->setSuffix(i18n("%"));
 
     connect(cmbPredefined, SIGNAL(activated(int)), SLOT(predefinedClicked(int)));
     connect(doubleResolution, SIGNAL(valueChanged(double)),
@@ -107,16 +109,18 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
             this, SLOT(heightUnitChanged(int)));
     connect(doubleHeight, SIGNAL(valueChanged(double)),
             this, SLOT(heightChanged(double)));
+    connect(cmbResolutionUnit, SIGNAL(currentIndexChanged(int)),
+            this, SLOT(resolutionUnitChanged()));
 
 
     // Create image
     newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setText(i18n("&Create"));
+    KGuiItem::assign(newDialogConfirmationButtonBox->button(QDialogButtonBox::Cancel), KStandardGuiItem::cancel());
     connect(newDialogConfirmationButtonBox, SIGNAL(accepted()), this, SLOT(createImage()));
 
 
     // Cancel Create image button
-    connect(newDialogConfirmationButtonBox, SIGNAL(rejected()), this->parentWidget(), SLOT(close()));
-    connect(newDialogConfirmationButtonBox, SIGNAL(rejected()), this->parentWidget(), SLOT(deleteLater()));
+    connect(newDialogConfirmationButtonBox, SIGNAL(rejected()), m_openPane, SLOT(reject()));
 
 
 
@@ -134,11 +138,6 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     colorSpaceSelector->setCurrentProfile(defColorProfile);
     connect(colorSpaceSelector, SIGNAL(colorSpaceChanged(const KoColorSpace*)), this, SLOT(changeDocumentInfoLabel()));
 
-    //connect(chkFromClipboard,SIGNAL(stateChanged(int)),this,SLOT(clipboardDataChanged()));
-    connect(QApplication::clipboard(), SIGNAL(dataChanged()), this, SLOT(clipboardDataChanged()));
-    connect(QApplication::clipboard(), SIGNAL(selectionChanged()), this, SLOT(clipboardDataChanged()));
-    connect(QApplication::clipboard(), SIGNAL(changed(QClipboard::Mode)), this, SLOT(clipboardDataChanged()));
-
     connect(colorSpaceSelector, SIGNAL(selectionChanged(bool)), newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
 
 
@@ -186,7 +185,7 @@ KisCustomImageWidget::~KisCustomImageWidget()
 }
 
 void KisCustomImageWidget::resolutionChanged(double res)
-{
+{  
     if (m_widthUnit.type() == KoUnit::Pixel) {
         m_widthUnit.setFactor(res / 72.0);
         m_width = m_widthUnit.fromUserValue(doubleWidth->value());
@@ -199,6 +198,21 @@ void KisCustomImageWidget::resolutionChanged(double res)
     changeDocumentInfoLabel();
 }
 
+void KisCustomImageWidget::resolutionUnitChanged()
+{
+    qreal resolution = doubleResolution->value();
+
+    if (cmbResolutionUnit->currentText() == pixelsInchStr()) {
+        resolution = KoUnit::convertFromUnitToUnit(resolution, KoUnit(KoUnit::Inch), KoUnit(KoUnit::Centimeter));
+    } else {
+        resolution = KoUnit::convertFromUnitToUnit(resolution, KoUnit(KoUnit::Centimeter), KoUnit(KoUnit::Inch));
+    }
+
+    {
+        KisSignalsBlocker b(doubleResolution);
+        doubleResolution->setValue(resolution);
+    }
+}
 
 void KisCustomImageWidget::widthUnitChanged(int index)
 {
@@ -255,6 +269,7 @@ void KisCustomImageWidget::createImage()
     if (doc) {
         doc->setModified(false);
         emit m_openPane->documentSelected(doc);
+        m_openPane->accept();
     }
 }
 
@@ -294,9 +309,15 @@ KisDocument* KisCustomImageWidget::createNewImage()
     KisDocument *doc = static_cast<KisDocument*>(KisPart::instance()->createDocument());
 
     qint32 width, height;
-    double resolution;
-    resolution = doubleResolution->value() / 72.0;  // internal resolution is in pixels per pt
-
+    double resolution ;
+    KIS_SAFE_ASSERT_RECOVER(cmbResolutionUnit->currentText() == pixelsInchStr() || cmbResolutionUnit->currentText() == pixelsCentimeterStr()) { resolution = 1.0; }
+    if (cmbResolutionUnit->currentText() == pixelsInchStr()) {
+        resolution = doubleResolution->value() / 72.0;  // internal resolution is in pixels per pt
+    } else if (cmbResolutionUnit->currentText() == pixelsCentimeterStr()) {
+        resolution = doubleResolution->value() / 28.34;
+    } else {
+        resolution = 1.0;
+    }
     width = static_cast<qint32>(0.5  + KoUnit(KoUnit::Pixel, resolution).toUserValuePrecise(m_width));
     height = static_cast<qint32>(0.5 + KoUnit(KoUnit::Pixel, resolution).toUserValuePrecise(m_height));
 
@@ -341,10 +362,6 @@ void KisCustomImageWidget::setBackgroundOpacity(quint8 value) {
   sliderOpacity->setValue((value * 100) / 255);
 }
 
-void KisCustomImageWidget::clipboardDataChanged()
-{
-}
-
 void KisCustomImageWidget::fillPredefined()
 {
     cmbPredefined->clear();
@@ -352,7 +369,7 @@ void KisCustomImageWidget::fillPredefined()
 
     cmbPredefined->addItem("");
 
-    QStringList definitions = KoResourcePaths::findAllResources("data", "predefined_image_sizes/*.predefinedimage", KoResourcePaths::Recursive);
+    QStringList definitions = KoResourcePaths::findAllAssets("data", "predefined_image_sizes/*.predefinedimage", KoResourcePaths::Recursive);
     definitions.sort();
 
     if (!definitions.empty()) {
@@ -388,7 +405,11 @@ void KisCustomImageWidget::predefinedClicked(int index)
 
     KisPropertiesConfigurationSP predefined = m_predefined[index - 1];
     txtPredefinedName->setText(predefined->getString("name"));
-    doubleResolution->setValue(predefined->getDouble("resolution"));
+    if (cmbResolutionUnit->currentText() == pixelsInchStr()) {
+        doubleResolution->setValue(predefined->getDouble("resolution"));
+    } else {
+        doubleResolution->setValue(KoUnit::convertFromUnitToUnit(predefined->getDouble("resolution"), KoUnit(KoUnit::Centimeter), KoUnit(KoUnit::Inch)));
+    }
     cmbWidthUnit->setCurrentIndex(predefined->getInt("x-unit"));
     cmbHeightUnit->setCurrentIndex(predefined->getInt("y-unit"));
 
