@@ -30,6 +30,8 @@ export CMAKE_PREFIX_PATH=$DEPS_INSTALL_PREFIX:$CMAKE_PREFIX_PATH
 export PYTHONPATH=$DEPS_INSTALL_PREFIX/sip
 export PYTHONHOME=$DEPS_INSTALL_PREFIX
 
+source ${KRITA_SOURCES}/packaging/linux/appimage/override_compiler.sh.inc
+
 cd $KRITA_SOURCES
 
 if [ -z "${KRITA_BRANDING}" ]; then
@@ -65,6 +67,7 @@ cmake $KRITA_SOURCES \
     -DHIDE_SAFE_ASSERTS=ON \
     -DFETCH_TRANSLATIONS=ON \
     -DBUILD_TESTING=FALSE \
+    -DKRITA_ENABLE_PCH=off \
     -DPYQT_SIP_DIR_OVERRIDE=$DEPS_INSTALL_PREFIX/share/sip/ \
     -DHAVE_MEMORY_LEAK_TRACKER=FALSE \
     -DBRANDING="${KRITA_BRANDING}"
