@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2018 Anna Medonosova <anna.medonosova@gmail.com>
+ *  SPDX-FileCopyrightText: 2018 Anna Medonosova <anna.medonosova@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "KisGamutMaskChooser.h"
@@ -23,17 +11,16 @@
 #include <QAbstractItemDelegate>
 #include <QMenu>
 #include <QActionGroup>
-#include <QToolButton>
 #include <QFontMetrics>
 #include <QTextDocument>
 #include <QTextLayout>
 
-#include <KoResourceServer.h>
-#include <KoResourceServerProvider.h>
-#include <KoResourceItemChooser.h>
-#include <KoResourceServerAdapter.h>
+#include <KisResourceItemChooser.h>
+#include <KisResourceItemListView.h>
+#include <KisResourceModel.h>
 #include <kis_icon_utils.h>
 #include <kis_config.h>
+#include "KisPopupButton.h"
 
 /// The resource item delegate for rendering the resource preview
 class KisGamutMaskDelegate: public QAbstractItemDelegate
@@ -67,14 +54,8 @@ void KisGamutMaskDelegate::paint(QPainter * painter, const QStyleOptionViewItem
     if (!index.isValid())
         return;
 
-    KoResource* resource = static_cast<KoResource*>(index.internalPointer());
-    KoGamutMask* mask = static_cast<KoGamutMask*>(resource);
-
-    if (!mask) {
-        return;
-    }
-
-    QImage preview = mask->image();
+    QImage preview = index.data(Qt::UserRole + KisAbstractResourceModel::Thumbnail).value<QImage>();
+    QString name = index.data(Qt::UserRole + KisAbstractResourceModel::Name).value<QString>();
 
     if(preview.isNull()) {
         return;
@@ -105,6 +86,11 @@ void KisGamutMaskDelegate::paint(QPainter * painter, const QStyleOptionViewItem
         QImage previewHighDPI = preview.scaled(previewSize*devicePixelRatioF, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
         previewHighDPI.setDevicePixelRatio(devicePixelRatioF);
 
+        if (option.state & QStyle::State_Selected) {
+            painter->setPen(QPen(option.palette.highlight(), 2.0));
+            painter->fillRect(option.rect, option.palette.highlight());
+            painter->setPen(option.palette.highlightedText().color());
+        }
         painter->drawImage(paintRect.x(), paintRect.y(), previewHighDPI);
 
         int leftMargin = 8;
@@ -118,11 +104,13 @@ void KisGamutMaskDelegate::paint(QPainter * painter, const QStyleOptionViewItem
                        QPointF(paintRect.width() - rightMargin, paintRect.y() + descOffset + painter->fontMetrics().lineSpacing()));
         painter->drawText(titleRect, Qt::AlignLeft,
                           painter->fontMetrics().elidedText(
-                              mask->title(), Qt::ElideRight, titleRect.width()
+                              name, Qt::ElideRight, titleRect.width()
                               )
                           );
-
-        if (!mask->description().isEmpty() && !mask->description().isNull()) {
+        QMap<QString, QVariant> metaData = index.data(Qt::UserRole + KisAbstractResourceModel::MetaData).value<QMap<QString, QVariant>>();
+        QString descriptionKey = "description";
+        QString description = metaData.contains(descriptionKey) ? metaData[descriptionKey].toString() : "";
+        if (!description.isEmpty() && !description.isNull()) {
             font.setPointSize(font.pointSize()-1);
             font.setBold(false);
             font.setStyle(QFont::StyleItalic);
@@ -133,7 +121,7 @@ void KisGamutMaskDelegate::paint(QPainter * painter, const QStyleOptionViewItem
             int numLines = floor(((float)descRect.height() / (float)painter->fontMetrics().lineSpacing()));
             if (numLines > 0) {
                 int elideWidth = 0;
-                QTextLayout textLayout(mask->description());
+                QTextLayout textLayout(description);
                 textLayout.beginLayout();
                 for (int i = 0; i < numLines; i++) {
                     QTextLine line = textLayout.createLine();
@@ -143,7 +131,7 @@ void KisGamutMaskDelegate::paint(QPainter * painter, const QStyleOptionViewItem
                     }
                 }
 
-                QString elidedText = painter->fontMetrics().elidedText(mask->description(), Qt::ElideRight, elideWidth);
+                QString elidedText = painter->fontMetrics().elidedText(description, Qt::ElideRight, elideWidth);
                 painter->drawText(descRect, Qt::AlignLeft|Qt::TextWordWrap, elidedText);
             }
         }
@@ -157,13 +145,10 @@ KisGamutMaskChooser::KisGamutMaskChooser(QWidget *parent) : QWidget(parent)
 {
     m_delegate = new KisGamutMaskDelegate(this);
 
-    KoResourceServer<KoGamutMask>* rServer = KoResourceServerProvider::instance()->gamutMaskServer();
-    QSharedPointer<KoAbstractResourceServerAdapter> adapter(new KoResourceServerAdapter<KoGamutMask>(rServer));
-    m_itemChooser = new KoResourceItemChooser(adapter, this);
+    m_itemChooser = new KisResourceItemChooser(ResourceType::GamutMasks, false, this);
     m_itemChooser->setItemDelegate(m_delegate);
     m_itemChooser->showTaggingBar(true);
     m_itemChooser->showButtons(false);
-    m_itemChooser->setColumnCount(4);
     m_itemChooser->setSynced(true);
 
     QVBoxLayout* layout = new QVBoxLayout(this);
@@ -194,13 +179,12 @@ KisGamutMaskChooser::KisGamutMaskChooser(QWidget *parent) : QWidget(parent)
     // setting the view mode
     setViewMode(m_mode);
     m_itemChooser->setViewModeButtonVisible(true);
-    QToolButton* viewModeButton = m_itemChooser->viewModeButton();
-    viewModeButton->setMenu(menu);
+    KisPopupButton* viewModeButton = m_itemChooser->viewModeButton();
+    viewModeButton->setPopupWidget(menu);
 
     layout->addWidget(m_itemChooser);
-    setLayout(layout);
 
-    connect(m_itemChooser, SIGNAL(resourceSelected(KoResource*)), this, SLOT(resourceSelected(KoResource*)));
+    connect(m_itemChooser, SIGNAL(resourceSelected(KoResourceSP )), this, SLOT(resourceSelected(KoResourceSP )));
 }
 
 KisGamutMaskChooser::~KisGamutMaskChooser()
@@ -208,7 +192,7 @@ KisGamutMaskChooser::~KisGamutMaskChooser()
 
 }
 
-void KisGamutMaskChooser::setCurrentResource(KoResource *resource)
+void KisGamutMaskChooser::setCurrentResource(KoResourceSP resource)
 {
     m_itemChooser->setCurrentResource(resource);
 }
@@ -232,19 +216,22 @@ void KisGamutMaskChooser::updateViewSettings()
 
     if (m_mode == KisGamutMaskChooser::THUMBNAIL) {
         m_itemChooser->setSynced(true);
+        m_itemChooser->itemView()->setViewMode(QListView::IconMode);
+        m_itemChooser->setRowHeight(this->fontMetrics().lineSpacing()*4);
+        m_itemChooser->setColumnWidth(this->fontMetrics().lineSpacing()*4);
         m_delegate->setViewMode(m_mode);
     } else if (m_mode == KisGamutMaskChooser::DETAIL) {
         m_itemChooser->setSynced(false);
-        m_itemChooser->setColumnCount(1);
+        m_itemChooser->itemView()->setViewMode(QListView::ListMode);
         m_itemChooser->setRowHeight(this->fontMetrics().lineSpacing()*4);
         m_itemChooser->setColumnWidth(m_itemChooser->width());
         m_delegate->setViewMode(m_mode);
     }
 }
 
-void KisGamutMaskChooser::resourceSelected(KoResource* resource)
+void KisGamutMaskChooser::resourceSelected(KoResourceSP resource)
 {
-    emit sigGamutMaskSelected(static_cast<KoGamutMask*>(resource));
+    emit sigGamutMaskSelected(resource.staticCast<KoGamutMask>());
 }
 
 void KisGamutMaskChooser::slotSetModeThumbnail()
