@@ -36,16 +36,7 @@
 
 OverviewDockerDock::OverviewDockerDock()
     : QDockWidget(i18n("Overview"))
-    , m_zoomSlider(nullptr)
-    , m_rotateAngleSelector(nullptr)
-    , m_mirrorCanvas(nullptr)
-    , m_pinControlsButton(nullptr)
-    , m_canvas(nullptr)
-    , m_cursorIsHover(false)
-    , m_isTouching(false)
-    , m_isDraggingWithTouch(false)
     , m_lastOverviewMousePos(0.0, 0.0)
-    , m_cumulatedMouseDistanceSquared(0.0)
 {
     m_page = new QWidget(this);
 
@@ -155,6 +146,7 @@ void OverviewDockerDock::setCanvas(KoCanvasBase * canvas)
             }
         }
         m_mirrorCanvas->setIcon(KisIconUtils::loadIcon("mirror-view-16"));
+        m_mirrorCanvas->setAutoRaise(true);
         connect(m_mirrorCanvas, SIGNAL(toggled(bool)), this, SLOT(mirrorUpdateIcon()));
 
         m_pinControlsButton = new QToolButton;
@@ -164,6 +156,7 @@ void OverviewDockerDock::setCanvas(KoCanvasBase * canvas)
             i18nc("Make the controls in the overview docker auto-hide or always visible", "Pin navigation controls")
         );
         m_pinControlsButton->setIcon(KisIconUtils::loadIcon("krita_tool_reference_images"));
+        m_pinControlsButton->setAutoRaise(true);
         connect(m_pinControlsButton, SIGNAL(toggled(bool)), SLOT(setPinControls(bool)));
 
         m_controlsSecondRowLayout->addWidget(m_rotateAngleSelector);
