@@ -43,6 +43,9 @@ typedef const struct _convertParams {
   const rgba *lookup ;
 } convertParams ;
 #define RGB_SHIFT RED_SHIFT, GREEN_SHIFT, BLUE_SHIFT
+#ifdef OPAQUE
+#undef OPAQUE
+#endif
 #define OPAQUE (255 << ALPHA_SHIFT)
 static convertParams convertRGB       = { 3, {RGB_SHIFT}, OPAQUE, 0 };
 static convertParams convertRGBA      = { 4, {RGB_SHIFT, ALPHA_SHIFT}, 0,0 };
@@ -121,8 +124,10 @@ initTileDirectory(struct tileDimensions *dim,struct xcfTiles *tiles,
    */
   data = xcfL(ptr) ;
   if( xcfL(ptr) != tiles->params->bpp ) {
-    FatalBadXCF("%"PRIu32" bytes per pixel for %s drawable",xcfL(ptr),type);
-    return XCF_ERROR;
+      FatalBadXCF("%" PRIuPTR " bytes per pixel for %s drawable",
+                  xcfL(ptr),
+                  type);
+      return XCF_ERROR;
   }
   uint32_t ptrout;
   if(xcfOffset(ptr+4,3*4, &ptrout) != XCF_OK) return XCF_ERROR;
