@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2012 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2012 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "kis_dab_cache.h"
@@ -83,6 +71,8 @@ KisFixedPaintDeviceSP KisDabCache::fetchDab(const KoColorSpace *cs,
         QRect *dstDabRect,
         qreal lightnessStrength)
 {
+    //Q_UNUSED(lightnessStrength);
+
     return fetchDabCommon(cs, colorSource, KoColor(),
                           cursorPoint,
                           shape,
@@ -110,6 +100,23 @@ KisFixedPaintDeviceSP KisDabCache::fetchDab(const KoColorSpace *cs,
                           lightnessStrength);
 }
 
+KisFixedPaintDeviceSP KisDabCache::fetchNormalizedImageDab(const KoColorSpace *cs,
+                                                           const QPointF &cursorPoint,
+                                                           KisDabShape const& shape,
+                                                           const KisPaintInformation& info,
+                                                           qreal softnessFactor,
+                                                           QRect *dstDabRect)
+{
+    return fetchDabCommon(cs, 0, KoColor(),
+                          cursorPoint,
+                          shape,
+                          info,
+                          softnessFactor,
+                          dstDabRect,
+                          1.0,
+                          true);
+}
+
 inline
 KisFixedPaintDeviceSP KisDabCache::fetchFromCache(KisDabCacheUtils::DabRenderingResources *resources,
                                                   const KisPaintInformation& info,
@@ -151,9 +158,11 @@ KisFixedPaintDeviceSP KisDabCache::fetchDabCommon(const KoColorSpace *cs,
         const KisPaintInformation& info,
         qreal softnessFactor,
         QRect *dstDabRect,
-        qreal lightnessStrength)
+        qreal lightnessStrength,
+        bool forceNormalizedRGBAImageStamp)
 {
     Q_ASSERT(dstDabRect);
+    //Q_UNUSED(lightnessStrength);
 
     bool hasDabInCache = true;
 
@@ -208,7 +217,7 @@ KisFixedPaintDeviceSP KisDabCache::fetchDabCommon(const KoColorSpace *cs,
 
     // 3. Generate new dab
 
-    generateDab(di, &resources, &m_d->dab);
+    generateDab(di, &resources, &m_d->dab, forceNormalizedRGBAImageStamp);
 
     // 4. Do postprocessing
     if (di.needsPostprocessing) {
