@@ -6,6 +6,7 @@
  */
 
 #include "kis_dynamic_sensors.h"
+#include "kis_config.h"
 
 
 KisDynamicSensorSpeed::KisDynamicSensorSpeed()
@@ -13,19 +14,6 @@ KisDynamicSensorSpeed::KisDynamicSensorSpeed()
 {
 }
 
-qreal KisDynamicSensorSpeed::value(const KisPaintInformation& info)
-{
-    /**
-     * The value of maximum speed was measured empirically. This is
-     * the speed that is quite easy to get with an A6 tablet and quite
-     * a big image. If you need smaller speeds, just change the curve.
-     */
-    const qreal maxSpeed = 30.0; // px / ms
-    const qreal currentSpeed = qMin(1.0, info.drawingSpeed() / maxSpeed);
-
-    return currentSpeed;
-}
-
 KisDynamicSensorRotation::KisDynamicSensorRotation()
     : KisDynamicSensor(ROTATION)
 {
