@@ -1,5 +1,6 @@
 /*
  *  SPDX-FileCopyrightText: 2017 Eugene Ingerman
+ *  SPDX-FileCopyrightText: 2022 L. E. Segovia <amy@amyspark.me>
  *
  *  SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -15,8 +16,8 @@
 #include <kis_signals_blocker.h>
 #include <kis_signal_compressor.h>
 #include <KisReferenceImage.h>
+#include <kis_clipboard.h>
 
-#include <QClipboard>
 #include <QApplication>
 #include <QStandardItemModel>
 
@@ -28,8 +29,8 @@ struct ToolReferenceImagesWidget::Private {
     {
     }
 
-    Ui_WdgToolOptions *ui;
-    ToolReferenceImages *tool;
+    Ui_WdgToolOptions *ui {nullptr};
+    ToolReferenceImages *tool {nullptr};
 };
 
 ToolReferenceImagesWidget::ToolReferenceImagesWidget(ToolReferenceImages *tool, KisCanvasResourceProvider */*provider*/, QWidget *parent)
@@ -77,10 +78,15 @@ ToolReferenceImagesWidget::ToolReferenceImagesWidget(ToolReferenceImages *tool,
     d->ui->bnPasteReferenceImage->setToolTip(i18n("Paste Reference Image From System Clipboard"));
     d->ui->bnPasteReferenceImage->setIcon(KisIconUtils::loadIcon("edit-paste-16"));
     d->ui->bnPasteReferenceImage->setIconSize(QSize(16, 16));
+    d->ui->bnPasteReferenceImage->setEnabled(KisClipboard::instance()->hasClip() || KisClipboard::instance()->hasUrls());
 
     connect(d->ui->bnAddReferenceImage, SIGNAL(clicked()), tool, SLOT(addReferenceImage()));
     connect(d->ui->bnPasteReferenceImage, SIGNAL(clicked()), tool, SLOT(pasteReferenceImage()));
 
+    connect(KisClipboard::instance(), &KisClipboard::clipChanged, this, [&]() {
+        d->ui->bnPasteReferenceImage->setEnabled(KisClipboard::instance()->hasClip() || KisClipboard::instance()->hasUrls());
+    });
+
     connect(d->ui->bnDelete, SIGNAL(clicked()), tool, SLOT(removeAllReferenceImages()));
     connect(d->ui->bnSave, SIGNAL(clicked()), tool, SLOT(saveReferenceImages()));
     connect(d->ui->bnLoad, SIGNAL(clicked()), tool, SLOT(loadReferenceImages()));
@@ -115,7 +121,7 @@ void ToolReferenceImagesWidget::selectionChanged(KoSelection *selection)
     bool anyNotKeepingAspectRatio = false;
     bool anyEmbedded = false;
     bool anyLinked = false;
-    bool anyNonLinkable = false;
+    // bool anyNonLinkable = false;
     bool anySelected = selection->count() > 0;
 
     Q_FOREACH(KoShape *shape, shapes) {
@@ -127,7 +133,7 @@ void ToolReferenceImagesWidget::selectionChanged(KoSelection *selection)
         if (reference) {
             anyEmbedded |= reference->embed();
             anyLinked |= !reference->embed();
-            anyNonLinkable |= !reference->hasLocalFile();
+            // anyNonLinkable |= !reference->hasLocalFile();
         }
     }
 
